﻿<#
    .SYNOPSIS
    Registers Customer code in QP
    .DESCRIPTION
    Registers new customer code in QP:
    - Restores database from backup
    - Brings database up-to-date
    - Registers customer code in QP confirutation file

    .EXAMPLE
    .\InstallCustomerCode.ps1 -databaseServer 'dbhost' -targetBackupPath 'c:\temp\qa_demosite.bak' -customerCode 'qa_demosite' -customerLogin 'login' -customerPassword '1q2w#E$R' 
    -currentSqlPath '\\storage\Developers_share\QP\current.sql' -adminPort 8021 -stagePort 8024
#>
param(
    ## Database Server
    [Parameter(Mandatory = $true)]
    [string] $databaseServer,
    ## Backup file for copying onto database Server
    [Parameter()]
    [ValidateScript({ if ($_) { Test-Path $_} })]
    [string] $sourceBackupPath,
    ## Backup file for restoring (server local - for SQL Server)
    [Parameter(Mandatory = $true)]
    [string] $targetBackupPath,
    ## Catalog customer code
    [Parameter(Mandatory = $true)]
    [string] $customerCode,
    ## Sql script for bringing DB up-to-date
    [Parameter(Mandatory = $true)]
    [string] $currentSqlPath,
    ## Customer user name
    [Parameter()]
    [String] $customerLogin,
    ## Customer user password
    [Parameter()]
    [String] $customerPassword,
    ## Reset user password
    [Parameter()]
    [bool] $resetUserPassword = $false,
    ## Admin user name
    [Parameter()]
    [String] $login,
    ## Admin user password
    [Parameter()]
    [String] $password,
    ## Database type: 0 - SQL Server, 1 - Postgres
    [Parameter()]
    [int] $dbType,
    ## Admin port
    [Parameter()]
    [int] $adminPort,
    ## Stage site port
    [Parameter()]
    [int] $stagePort
)

function PSqlToPsObject
{
    param(
        $lines
    )

    $result = New-Object PsObject
    foreach ($line in $lines) {
        $values = $line.Split('|')
        if ($values.Length -lt 2) {
            continue
        }
        $result | Add-Member NoteProperty $values[0] $values[1]
    }
    return $result

}


function GetFieldId
{
    param(
        [hashtable] $connectionParams,
        [string] $key,
        [string] $field
    )
    
    $isPg = $connectionParams.dbType -eq 1
    $keyField = if ($isPg) { "Key" } else { "[Key]" }
    $query = "select value from APP_SETTINGS where $keyField = '$key'"
    $result = Execute-Sql @connectionParams -Query $query  
    $contentId = if ($isPg) { (PSqlToPsObject -lines $result).Value } else { $result.Value }
    
    if (!$contentId) { throw "setting '$key' is not exists"}
    $query = "select ATTRIBUTE_ID from CONTENT_ATTRIBUTE where CONTENT_ID = $contentId and ATTRIBUTE_NAME = '$field'"
    $result = Execute-Sql @connectionParams -Query $query  
    $fieldId = if ($isPg) { (PSqlToPsObject -lines $result).ATTRIBUTE_ID } else { $result.ATTRIBUTE_ID } 

    if (!$fieldId) { throw "field '$field' is not found for content $contentId"}
    return $fieldId
}

function ReplaceFieldValues
{
    param(
        [hashtable] $connectionParams,
        [int] $fieldId,
        [string] $placeholder,
        [string] $value
    )  

    $query = "update content_data set data = replace(data, '$placeholder', '$value') where ATTRIBUTE_ID = $fieldId and data like '%$placeholder%'"
    Execute-Sql @connectionParams -Query $query   
}

$currentPath = Split-path -parent $MyInvocation.MyCommand.Definition

. (Join-Path $currentPath "Modules\Database.ps1")
. (Join-Path $currentPath "Modules\CustomerCode.ps1")

if (Get-CustomerCode -CustomerCode $customerCode)
{
    Write-Host "Customer code $customerCode already exists"
    return
}

if (-not $customerLogin){
    $resetUserPassword = $true
    $customerLogin = "consolidation_${customerCode}_login"
}
if (-not $customerPassword){

    $customerPassword = New-Guid
}

if (-not [string]::IsNullOrEmpty($sourceBackupPath))
{
    if ($dbType -eq 0) {
        $sharedTargetBackupPath =  "\\" + $databaseServer.Trim() + "\" + $targetBackupPath.Replace(":", "$")
    } else {
        $sharedTargetBackupPath = $targetBackupPath
    }
    Write-Host "Copy backup from $sourceBackupPath to $sharedTargetBackupPath"
    
    try
    {
        Copy-Item $sourceBackupPath -Destination $sharedTargetBackupPath -ErrorAction Stop
    }
    catch
    {
        throw $_.Exception
    }

    Write-Host "Backup copied" 
}

$dbParams = @{
    DatabaseServer = $databaseServer;
    DatabaseName = $customerCode;
    Login = $login;
    Password = $password;
    DbType = $dbType
}

Restore-Database @dbParams -BackupPath $targetBackupPath  
Add-DatabaseUser @dbParams -UserName $customerLogin -UserPassword $customerPassword -ResetUserPassword $resetUserPassword 

$cnnParams = @{
    Server = $databaseServer;
    Database = $customerCode;
    Name = $login;
    Pass = $password;
    DbType = $dbType;
}

Write-Host "Run Current.sql on $currentSqlPath"  
Execute-File @cnnParams -path $currentSqlPath 
Write-Host "Current.sql updated"  

if ($adminPort -ne 0 -and $stagePort -ne 0)
{
    Write-Host "Update database"  

    $adminUrl = "http://${env:COMPUTERNAME}:$adminPort"
    $stageUrl = "http://${env:COMPUTERNAME}:$stagePort/blog"
    
    $updateQuery = "update custom_action set url = '$adminUrl' where name = 'Manage Pages'"
    Execute-Sql @cnnParams -query $updateQuery 
    
    $updateQuery2 = "update custom_action set url = '$stageUrl' where name = 'OnScreen'"
    Execute-Sql @cnnParams -query $updateQuery2 
     
    Write-Host "Database updated"
}
  
$savedParams = @{
    Server = $databaseServer;
    Database = $customerCode;
    Name = $customerLogin;
    Pass = $customerPassword;
    DbType = $dbType;
}

$connectionString = Get-ConnectionString @savedParams -forConfig $true
Add-CustomerCode -CustomerCode $customerCode -ConnectionString $connectionString -dbType $dbType

Apply-QPConfigurationPathx64