﻿<#
    .SYNOPSIS
    Creates QP customer code

    .DESCRIPTION
    During installation script does the following actions:
    - Validates connection to database Server
    - Restores sample database from backup
    - Updates sample database to actual state
    - Registers sample database as QP customer code

    .EXAMPLE
    .\InstallCustomerCodeOnly.ps1 -databaseServer dbhost -customerCode code

#>
param (
    [Parameter(Mandatory = $true)]
    [string] $databaseServer,
    ## Backup file for copying onto database server
    [Parameter()]
    [ValidateScript({ if ($_) { Test-Path $_} })]
    [string] $sourceBackupPath,
    ## Backup file for restoring (server local - for SQL Server)
    [Parameter()]
    [string] $targetBackupPath,
    ## Admin database user name
    [Parameter()]
    [String] $login,
    ## Admin database user password
    [Parameter()]
    [String] $password,
    ## Sample database user name
    [Parameter()]
    [string] $customerLogin,
    ## Sample database user password
    [Parameter()]
    [string] $customerPassword,
    ## Path to sql script for bringing installed database up-to-date
    [Parameter()]
    [ValidateScript({ if ($_) { Test-Path $_} })]
    [string] $currentSqlPath,
    ## Customer code
    [Parameter(Mandatory = $true)]
    [string] $customerCode,
    ## Administration site name
    [Parameter()]
    [int] $dbType = 0
)

If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
{   
    $arguments = "& '" + $myinvocation.mycommand.definition + "'"
    Start-Process powershell -Verb runAs -ArgumentList $arguments
    Break
}


$currentPath = Split-path -parent $MyInvocation.MyCommand.Definition

if (!$currentSqlPath) {
    $path = Join-Path $currentPath "current.sql"
    if (Test-Path $path){
        $currentSqlPath = $path
    } else { 
        throw "currentSqlPath is not found on $path"
    }
}


$defaultBackupName = if ($dbType -eq 0) { "qa_demosite.bak" } else { "qa_demosite.dump" }
if (!$sourceBackupPath) {
    $path = Join-Path $currentPath $defaultBackupName
    if (Test-Path $path){
        $sourceBackupPath = $path;
    }
}

if (!$targetBackupPath) {
    $targetBackupPath = "C:\temp\$defaultBackupName"
}

. (Join-Path $currentPath "Modules\Database.ps1")
. (Join-Path $currentPath "Modules\CustomerCode.ps1")

If ($databaseServer) {
    $useSqlPs = (-not(Get-Module -ListAvailable -Name SqlServer))
    $moduleName = if ($useSqlPs) { "SqlPS" } else { "SqlServer" }
    if (-not(Get-Module -Name $moduleName)) {
        Import-Module $moduleName
    }

    Try {
        Execute-Sql -server $databaseServer -name $login -pass $password -query "select 1 as result" -dbType $dbType | Out-Null
    } Catch {
        Write-Error $_.Exception
        Throw "Test connection to  $databaseServer : failed"
    } 
}

$resetUserPassword = $false
if (-not $customerLogin){
    $resetUserPassword = $true
    $customerLogin = "qp_${customerCode}_login"
}
if (-not $customerPassword){
    $customerPassword = New-Guid
}

$scriptName = Join-Path $currentPath "InstallCustomerCode.ps1"
$params = "-DatabaseServer '$databaseServer' -TargetBackupPath '$targetBackupPath' -CustomerCode '$customerCode' -CustomerLogin '$customerLogin' -CustomerPassword '$customerPassword' -CurrentSqlPath '$currentSqlPath' -DbType $dbType -resetUserPassword `$$resetUserPassword -adminPort 0 -stagePort 0"
if ($sourceBackupPath) { 
    $params = "$params -SourceBackupPath '$sourceBackupPath'" 
}
if ($login -and $password) {
    $params = "$params  -Login '$login' -Password '$password'" 
}
Invoke-Expression "$scriptName $params"